-- $Id: Core.lua 120 2021-05-19 13:30:24Z arithmandar $
-----------------------------------------------------------------------
-- Upvalued Lua API.
-----------------------------------------------------------------------
-- Functions
local _G = getfenv(0)
local select = _G.select
local pairs = _G.pairs
local math = _G.math
local ceil = math.ceil
-- Libraries
local GameTooltip, GetAddOnInfo, GetAddOnEnableState, UnitName, PlaySound, GetCVarBool, SetCVar = _G.GameTooltip, _G.GetAddOnInfo, _G.GetAddOnEnableState, _G.UnitName, _G.PlaySound, _G.GetCVarBool, _G.SetCVar
local WorldMapTrackingOptionsDropDown_OnClick
local GetBuildInfo = _G.GetBuildInfo


-- Determine WoW TOC Version
local WoWClassic, WoWRetail
local wowtocversion  = select(4, GetBuildInfo())
if wowtocversion < 30000 then
	WoWClassic = true
else
	WoWRetail = true
end

if WoWClassic then

else
	WorldMapTrackingOptionsDropDown_OnClick = _G.WorldMapTrackingOptionsDropDown_OnClick
end

-- ----------------------------------------------------------------------------
-- AddOn namespace.
-- ----------------------------------------------------------------------------
local FOLDER_NAME, private = ...

local LibStub = _G.LibStub
local L = LibStub("AceLocale-3.0"):GetLocale(private.addon_name)
local AceDB = LibStub("AceDB-3.0")
-- UIDropDownMenu
local LibDD = LibStub:GetLibrary("LibUIDropDownMenu-4.0")

local addon = LibStub("AceAddon-3.0"):NewAddon(private.addon_name, "AceEvent-3.0")
addon.constants = private.constants
addon.constants.addon_name = private.addon_name
addon.Name = FOLDER_NAME
local _
_, addon.LocName, addon.Notes = GetAddOnInfo(addon.Name)
-- ToC Metadata
addon.Version 		= GetAddOnMetadata(addon.Name, "Version")
addon.UpdateDate 	= GetAddOnMetadata(addon.Name, "X-Date")
addon.Author 		= GetAddOnMetadata(addon.Name, "Author")
--addon.LocName = select(2, GetAddOnInfo(addon.Name))
--addon.Notes = select(3, GetAddOnInfo(addon.Name))

addon.plugins = {}
_G.WorldMapTrackingEnhanced = addon
local profile
local FilterButton -- WorldMapFrame.overlayFrames[2]
local iMaxMenu = 20 -- Maximum number of dropdown menu item

local function checkAddonStatus(addonName)
	if not addonName then return nil end
	local loadable = select(4, GetAddOnInfo(addonName))
	local enabled = GetAddOnEnableState(UnitName("player"), addonName)
	if (enabled > 0 and loadable) then
		return true
	else
		return false
	end
end

local function onSelection(value, checked)
	if WoWClassic then
		-- do nothing
	else
		if (checked) then
			PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON)
		else
			PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_OFF)
		end

		if (value == "quests") then
			SetCVar("questPOI", checked and "1" or "0", "QUEST_POI")
		elseif (value == "dungeon entrances") then
			SetCVar("showDungeonEntrancesOnMap", checked and "1" or "0", "SHOW_DUNGEON_ENTRANCES")
		elseif (value == "digsites") then
			SetCVar("digSites", checked and "1" or "0", "SHOW_DIG_SITES")
		elseif (value == "tamers") then
			SetCVar("showTamers", checked and "1" or "0", "SHOW_TAMERS")
		elseif (value == "primaryProfessionsFilter" or value == "secondaryProfessionsFilter") then
			SetCVar(value, checked and "1" or "0")
		elseif (value == "worldQuestFilterResources" or value == "worldQuestFilterArtifactPower" or
				value == "worldQuestFilterProfessionMaterials" or value == "worldQuestFilterGold" or
				value == "worldQuestFilterEquipment" or value == "worldQuestFilterReputation") then
			-- World quest reward filter cvars
			SetCVar(value, checked and "1" or "0")
		end
		FilterButton:GetParent():RefreshAllDataProviders()
	end
end

-- //////////////////////////////////////////////////////////////////////////
-- Main function to replace World Map Tracking Option's dropdown menu
-- //////////////////////////////////////////////////////////////////////////
local function dropDown_Initialize(self, level)
	local function OnSelection(button)
		onSelection(button.value, button.checked)
	end
	
	local function AddSeparator()
		if (profile.useSeparator) then
			LibDD:UIDropDownMenu_AddSeparator(1)
		end
	end

	if not level then level = 1 end
	local info = LibDD:UIDropDownMenu_CreateInfo()

	if (level == 1) then
		info.isTitle = true
		info.notCheckable = true
		info.text = WORLD_MAP_FILTER_TITLE -- which is "Show:"
		LibDD:UIDropDownMenu_AddButton(info)

		info.isTitle = nil
		info.disabled = nil
		info.notCheckable = nil
		info.isNotRadio = true
		info.keepShownOnClick = true
		info.func = OnSelection

		if not WoWClassic then
			info.text = SHOW_QUEST_OBJECTIVES_ON_MAP_TEXT
			info.value = "quests"
			info.checked = GetCVarBool("questPOI")
			LibDD:UIDropDownMenu_AddButton(info)

			info.text = SHOW_DUNGEON_ENTRACES_ON_MAP_TEXT
			info.value = "dungeon entrances"
			info.checked = GetCVarBool("showDungeonEntrancesOnMap")
			LibDD:UIDropDownMenu_AddButton(info)

			local prof1, prof2, arch, fish, cook, firstAid = GetProfessions()
			if arch then
				info.text = ARCHAEOLOGY_SHOW_DIG_SITES -- "Show Digsites"
				info.value = "digsites"
				info.checked = GetCVarBool("digSites")
				LibDD:UIDropDownMenu_AddButton(info)
			end

			-- BZ's Pet Battle
			if CanTrackBattlePets() then
				info.text = SHOW_PET_BATTLES_ON_MAP_TEXT
				info.value = "tamers"
				info.checked = GetCVarBool("showTamers")
				LibDD:UIDropDownMenu_AddButton(info)
			end
		end
		-- If we aren't on a map with world quests don't show the world quest reward filter options.
		local mapID = WorldMapFrame:GetMapID()
		if mapID and MapUtil.MapHasEmissaries(mapID) then
			-- Adding World Quest Tracker menus;
--[[			if (checkAddonStatus("WorldQuestTracker") and profile.enable_WorldQuestTracker) then
				AddSeparator()
				local WQT = addon:GetModule("WorldQuestTracker", true)
				local menu = WQT:DropDownMenus()

				if (profile.worldQuestTracker_contextMenu) then
					LibDD:UIDropDownMenu_AddButton(menu[1])
				else
					for i = 1, #menu do
						LibDD:UIDropDownMenu_AddButton(menu[i])
					end
				end
			-- With World Quest Tracker enabled, actually the WoW built-in World Quest menu filters will not work. 
			else]]
				-- Clear out the info from the separator wholesale.
				info = LibDD:UIDropDownMenu_CreateInfo()

				info.isTitle = nil
				info.disabled = nil
				info.notCheckable = nil
				info.isNotRadio = true
				info.keepShownOnClick = true
				info.func = WorldMapTrackingOptionsDropDown_OnClick

				if prof1 or prof2 then
					info.text = SHOW_PRIMARY_PROFESSION_ON_MAP_TEXT
					info.value = "primaryProfessionsFilter"
					info.checked = GetCVarBool("primaryProfessionsFilter")
					LibDD:UIDropDownMenu_AddButton(info)
				end

				if fish or cook or firstAid then
					info.text = SHOW_SECONDARY_PROFESSION_ON_MAP_TEXT
					info.value = "secondaryProfessionsFilter"
					info.checked = GetCVarBool("secondaryProfessionsFilter")
					LibDD:UIDropDownMenu_AddButton(info)
				end
				
				AddSeparator()

				-- Clear out the info from the separator wholesale.
				info = LibDD:UIDropDownMenu_CreateInfo()

				info.isTitle = true
				info.notCheckable = true
				info.text = WORLD_QUEST_REWARD_FILTERS_TITLE
				LibDD:UIDropDownMenu_AddButton(info)
				info.text = nil

				info.isTitle = nil
				info.disabled = nil
				info.notCheckable = nil
				info.isNotRadio = true
				info.keepShownOnClick = true
				info.func = OnSelection

				info.text = WORLD_QUEST_REWARD_FILTERS_RESOURCES
				info.value = "worldQuestFilterResources"
				info.checked = GetCVarBool("worldQuestFilterResources")
				LibDD:UIDropDownMenu_AddButton(info)

				info.text = WORLD_QUEST_REWARD_FILTERS_ARTIFACT_POWER
				info.value = "worldQuestFilterArtifactPower"
				info.checked = GetCVarBool("worldQuestFilterArtifactPower")
				LibDD:UIDropDownMenu_AddButton(info)

				info.text = WORLD_QUEST_REWARD_FILTERS_PROFESSION_MATERIALS
				info.value = "worldQuestFilterProfessionMaterials"
				info.checked = GetCVarBool("worldQuestFilterProfessionMaterials")
				LibDD:UIDropDownMenu_AddButton(info)

				info.text = WORLD_QUEST_REWARD_FILTERS_GOLD
				info.value = "worldQuestFilterGold"
				info.checked = GetCVarBool("worldQuestFilterGold")
				LibDD:UIDropDownMenu_AddButton(info)
				
				info.text = WORLD_QUEST_REWARD_FILTERS_EQUIPMENT
				info.value = "worldQuestFilterEquipment"
				info.checked = GetCVarBool("worldQuestFilterEquipment")
				LibDD:UIDropDownMenu_AddButton(info)
				
				info.text = WORLD_QUEST_REWARD_FILTERS_REPUTATION
				info.value = "worldQuestFilterReputation"
				info.checked = GetCVarBool("worldQuestFilterReputation")
				LibDD:UIDropDownMenu_AddButton(info)
			--end
		end
		for k, v in pairs(addon.plugins) do
			if (v and profile.enableModules[k]) then
				if (k == "WorldQuestTracker") then
					-- do nothing
				else
					local Module = addon:GetModule(k, true)
					local menu = Module:DropDownMenus()
					
					AddSeparator()
					for i = 1, #menu do
						LibDD:UIDropDownMenu_AddButton(menu[i])
					end
				end
			end
		end
		-- Adding WorldMapTrackingEnhanced's Config link
		AddSeparator()
		info = LibDD:UIDropDownMenu_CreateInfo()
		info.isNotRadio = true
		info.notCheckable = true
		info.text = L["World Map Tracking Enhanced Config"]
		info.colorCode = "|cFFB5E61D"
		info.tooltipTitle = addon.LocName
		info.tooltipText = L["Click to open World Map Tracking Enhanced's config panel"]
		info.tooltipOnButton = true
		info.func = (function(self)
			ToggleFrame(WorldMapFrame)
			InterfaceOptionsFrame_OpenToCategory(addon.LocName)
			InterfaceOptionsFrame_OpenToCategory(addon.LocName)
		end)
		LibDD:UIDropDownMenu_AddButton(info)
	-- Handling level 2 menus
	elseif (level == 2) then
		for k, v in pairs(addon.plugins) do
			if (v and profile.enableModules[k] and L_UIDROPDOWNMENU_MENU_VALUE == k) then
				local Module = addon:GetModule(k, true)
				local _, menu2 = Module:DropDownMenus()
				
				if (menu2) then
					for i = 1, #menu2 do
						LibDD:UIDropDownMenu_AddButton(menu2[i], 2)
					end
				end
			end
		end
	elseif (level == 3) then
		for k, v in pairs(addon.plugins) do
			if (v and profile.enableModules[k]) then
				local Module = addon:GetModule(k, true)
				local _, menu2 = Module:DropDownMenus()
				
				if (menu2) then
					for i = 1, #menu2 do
						if (L_UIDROPDOWNMENU_MENU_VALUE == k..i) then
							local t = menu2[i].menuTable
							if (t) then
								for j = 1, #t do
									LibDD:UIDropDownMenu_AddButton(t[j], 3)
								end
							end
						end
					end
				end
			end
		end
	end
end

local function createTrackingButton()
	local parent
	
	local name = addon.Name.."Button"
	local f = _G[name]
	if WoWClassic then
		parent = WorldMapFrame
		parent.BlackoutFrame:Hide()
		parent:SetFrameStrata("HIGH")
		parent.BorderFrame:SetFrameStrata("LOW")

		if not f then f = CreateFrame("Button", name, parent) end
		f:SetWidth(32)
		f:SetHeight(32)
		f:SetFrameLevel(20)
		f:SetToplevel(true)
		f:ClearAllPoints()
		f:SetPoint("TOPRIGHT", parent, "TOPRIGHT", -8, -70)
	else
		parent = FilterButton:GetParent()
		
		if not f then f = CreateFrame("Button", name, WorldMapFrame.ScrollContainer) end
		f:SetWidth(32)
		f:SetHeight(32)
		f:SetPoint("TOPRIGHT", parent:GetCanvasContainer(), -4, -2, "TOPRIGHT")
	end
	
	addon.button = f
	
	
	f:SetFrameStrata("HIGH")

--[[	-- Layers
	f.Shadow = f:CreateTexture(name.."Shadow", "BACKGROUND")
	f.Shadow:SetAtlas("MapCornerShadow-Right", true)
	f.Shadow:SetPoint("TOPRIGHT", f, 4, 4)
	f.Shadow:SetTexCoord(0, 1, 1, 0)
	f.Shadow:SetDrawLayer("BACKGROUND", -1)

	f.Background = f:CreateTexture(name.."Background", "BACKGROUND")
	f.Background:SetPoint("TOPLEFT", f, 2, -4)
	f.Background:SetWidth(25)
	f.Background:SetHeight(25)
	f.Background:SetTexture("Interface\\Minimap\\UI-Minimap-Background")
	f.Background:SetVertexColor(1, 1, 1, 1)
]]
	f.Icon = f:CreateTexture(name.."Icon", "ARTWORK")
	f.Icon:SetHeight(20)
	f.Icon:SetWidth(20)
	f.Icon:SetPoint("TOPLEFT", f, 6, -6)
	f.Icon:SetTexture("Interface\\Minimap\\Tracking\\None")
	
	f.IconOverlay = f:CreateTexture(name.."IconOverlay", "OVERLAY")
	f.IconOverlay:Hide()
	f.IconOverlay:SetWidth(27)
	f.IconOverlay:SetHeight(27)
	f.IconOverlay:SetPoint("TOPLEFT", f.Icon, 0, 0)
	f.IconOverlay:SetPoint("BOTTOMRIGHT", f.Icon, 0, 0)
	f.IconOverlay:SetVertexColor(0, 0, 0, 0.5)

	f.Border = f:CreateTexture(name.."Border", "OVERLAY")
	f.Border:SetTexture("Interface\\Minimap\\MiniMap-TrackingBorder")
	f.Border:SetPoint("TOPLEFT", f, 0, 0)
	f.Border:SetWidth(54)
	f.Border:SetHeight(54)
	f.Border:SetDrawLayer("BACKGROUND", -1)

	f.DropDown = LibDD:Create_UIDropDownMenu(name.."DropDown", f)
	f.DropDown:SetClampedToScreen(true)
	f.DropDown:SetScript("OnShow", function(self) 
		LibDD:UIDropDownMenu_Initialize(self, dropDown_Initialize, "MENU") 
	end)
	
	f:SetHighlightTexture("Interface\\Minimap\\UI-Minimap-ZoomButton-Highlight", "ADD")
	
	f:SetScript("OnEnter", function(self)
		GameTooltip:SetOwner(self, "ANCHOR_TOPRIGHT")
		GameTooltip:SetText(TRACKING, 1, 1, 1)
		GameTooltip:AddLine(MINIMAP_TRACKING_TOOLTIP_NONE, nil, nil, nil, true)
		GameTooltip:Show()
	end)
	f:SetScript("OnLeave", function(self)
		GameTooltip:Hide()
	end)
	f:SetScript("OnClick", function(self)
		local parent = self:GetParent()
		local mapID = WorldMapFrame:GetMapID()
		if not mapID then
			return
		end
		self.DropDown.mapID = mapID or 0
		LibDD:ToggleDropDownMenu(1, nil, f.DropDown, f, 0, -5)
		PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON)
	end)
	f:SetScript("OnMouseDown", function(self)
		self.Icon:SetPoint("TOPLEFT", 8, -8)
		self.IconOverlay:Show()
	end)
	f:SetScript("OnMouseUp", function(self)
		self.Icon:SetPoint("TOPLEFT", 6, -6)
		self.IconOverlay:Hide()
	end)
	
end

function addon:OnInitialize()
	self.db = AceDB:New(addon.Name.."DB", addon.constants.defaults, true)
	profile = self.db.profile

	self.db.RegisterCallback(self, "OnProfileChanged", "Refresh")
	self.db.RegisterCallback(self, "OnProfileCopied", "Refresh")
	self.db.RegisterCallback(self, "OnProfileReset", "Refresh")

	self:SetupOptions()
	if WoWClassic then
		-- do nothing
	else
		FilterButton = WorldMapFrame.overlayFrames[2]
	end
end

local function rearrangeFilterButtonTexture()
	if WoWClassic then return end
	FilterButton.Background:SetTexture(nil)
	FilterButton.Icon:SetTexture(nil)
	FilterButton.IconOverlay:SetTexture(nil)
	FilterButton.Border:SetTexture(nil)
	FilterButton:SetHighlightTexture(nil)
end

function addon:OnEnable()
	for key, value in pairs( addon.constants.events ) do
		self:RegisterEvent( value )
	end
	createTrackingButton()
	rearrangeFilterButtonTexture()
--	FilterButton:Hide()
end

function addon:Refresh()
	profile = self.db.profile

	for k,v in self:IterateModules() do
		if self:GetModuleEnabled(k) and not v:IsEnabled() then
			self:EnableModule(k)
		elseif not self:GetModuleEnabled(k) and v:IsEnabled() then
			self:DisableModule(k)
		end
		if (v.Refresh) and (type(v.Refresh) == "function") then
			v:Refresh()
		end
	end
end

function addon:GetModuleEnabled(module)
	return profile.enableModules[module]
end

function addon:SetModuleEnabled(module, value)
	local old = profile.enableModules[module]
	profile.enableModules[module] = value
	if old ~= value then
		if value then
			self:EnableModule(module)
		else
			self:DisableModule(module)
		end
	end
end
